/**
 * @file 2x2_Key_click_AVR.c
 * @brief <h2> 2x2 Key click Example </h2>
 *
 * @par
 * Example for <a href="http://www.mikroe.com">MikroElektronika's</a>
 * 2x2 Key click board.
 *
 *
 *
 ******************************************************************************/

/**
 * @page        Example
 * @date        10 Nov 2016
 * @author      Branislav Marton
 * @copyright   GNU Public License
 * @version     1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :    ATMEGA32
 * -<b> Dev. Board    </b> :    EasyAVR v7
 * -<b> Oscillator    </b> :
 * -<b> Ext. Modules  </b> :    2x2 Key click board
 * -<b> SW            </b> :    mikroC PRO for AVR v6.1.2
 *
 */

/******************************************************************************
* Includes
*******************************************************************************/
#include <stdint.h>

/******************************************************************************
* Preprocessors Constants
*******************************************************************************/
#define X1 1
#define X2 2
#define Y1 4
#define Y2 12

#define PRESSED  0xFF
#define RELEASED 0x00

#define POLLING_PERIOD 50

/******************************************************************************
* Typedefs
*******************************************************************************/


/******************************************************************************
* Variable Definitions
*******************************************************************************/
sbit TAST1 at PINA7_bit;
sbit TAST1_DIR at DDA7_bit;

sbit TAST2 at PINA6_bit;
sbit TAST2_DIR at DDA6_bit;

sbit TAST3 at PINA5_bit;
sbit TAST3_DIR at DDA5_bit;

sbit TAST4 at PIND4_bit;
sbit TAST4_DIR at DDD4_bit;

// LCD module connections
sbit LCD_RS at PORTA2_bit;
sbit LCD_EN at PORTD6_bit;
sbit LCD_D4 at PORTC4_bit;
sbit LCD_D5 at PORTC5_bit;
sbit LCD_D6 at PORTC6_bit;
sbit LCD_D7 at PORTC7_bit;

sbit LCD_RS_Direction at DDA2_bit;
sbit LCD_EN_Direction at DDD6_bit;
sbit LCD_D4_Direction at DDC4_bit;
sbit LCD_D5_Direction at DDC5_bit;
sbit LCD_D6_Direction at DDC6_bit;
sbit LCD_D7_Direction at DDC7_bit;
// End LCD module connections

uint8_t t1_state = 0x00;
uint8_t t2_state = 0x00;
uint8_t t3_state = 0x00;
uint8_t t4_state = 0x00;

/******************************************************************************
* Function Prototypes
*******************************************************************************/

static void system_init( void );
static void port_init( void );

uint8_t Taster_Pressed(char taster, uint8_t* state);
uint8_t Taster_Released(char taster, uint8_t* state);

void Draw_Taster(
    unsigned int x, 
    unsigned int y, 
    char state, 
    unsigned char* label
);

/******************************************************************************
* Private Functions
*******************************************************************************/

/***************************************************************************//**
 *
 */
static void system_init( void )
{
    port_init();
    LCD_Init();
    Lcd_Cmd(_LCD_CLEAR);               // Clear display
    Lcd_Cmd(_LCD_CURSOR_OFF);  

    Lcd_Out(1, 1, "T1");
    Lcd_Out(1, 9, "T2");
    Lcd_Out(2, 1, "T3");
    Lcd_Out(2, 9, "T4");
}

/***************************************************************************//**
 *
 */
static void port_init( void )
{
    TAST1_DIR = 0;
    TAST2_DIR = 0;
    TAST3_DIR = 0;
    TAST4_DIR = 0;
}

/***************************************************************************//**
 *
 */
uint8_t Taster_Pressed(char taster, uint8_t* state)
{
    if(taster && *state == 0x00)
    {
        return 0xFF;
    }

    return 0x00;
}

/***************************************************************************//**
 *
 */
uint8_t Taster_Released(char taster, uint8_t* state)
{
    if(!taster && *state)
    {
        return 0xFF;
    }
    
    return 0x00;
}

/***************************************************************************//**
 *
 */
void Draw_Taster(
    unsigned int x, 
    unsigned int y, 
    char state, 
    unsigned char* label
)
{
    if(state)
        Lcd_Out(x, y, "ON ");

    else
        Lcd_Out(x, y, "OFF");

}

/******************************************************************************
* Public Functions
*******************************************************************************/
void main()
{
    system_init();

    Draw_Taster(X1, Y1, RELEASED, "T1");
    Draw_Taster(X2, Y1, RELEASED, "T2");
    Draw_Taster(X1, Y2, RELEASED, "T3");
    Draw_Taster(X2, Y2, RELEASED, "T4");

    while(1)
    {
        if(Taster_Pressed(TAST1, &t1_state))
            Draw_Taster(X1, Y1, PRESSED, "T1");

        if(Taster_Released(TAST1, &t1_state))
            Draw_Taster(X1, Y1, RELEASED, "T1");
            
        if(Taster_Pressed(TAST2, &t2_state))
            Draw_Taster(X1, Y2, PRESSED, "T2");

        if(Taster_Released(TAST2, &t2_state))
            Draw_Taster(X1, Y2, RELEASED, "T2");
            
        if(Taster_Pressed(TAST3, &t3_state))
            Draw_Taster(X2, Y1, PRESSED, "T3");

        if(Taster_Released(TAST3, &t3_state))
            Draw_Taster(X2, Y1, RELEASED, "T3");
            
        if(Taster_Pressed(TAST4, &t4_state))
            Draw_Taster(X2, Y2, PRESSED, "T4");

        if(Taster_Released(TAST4, &t4_state))
            Draw_Taster(X2, Y2, RELEASED, "T4");
            
        t1_state = TAST1;
        t2_state = TAST2;
        t3_state = TAST3;
        t4_state = TAST4;
            
        Delay_ms(POLLING_PERIOD);
    }
}
/*************** END OF FUNCTIONS *********************************************/